<?php

namespace Drupal\easy_email_override\Service;

use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;


/**
 * Class EmailManager.
 *
 * Decorates the doMail method to returning results without acutally
 * sending the very mail a module requests.
 *
 * @package Drupal\easy_email
 */
class EmailManager extends DefaultPluginManager implements MailManagerInterface {

  /**
   * Original service object.
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $originalService;


  /**
   * Constructs the EmailManager object.
   * @param \Drupal\Core\Mail\MailManagerInterface $original_service
   */
  public function __construct(MailManagerInterface $original_service) {
    $this->originalService = $original_service;
  }

  /**
   * @inheritDoc
   */
  public function getInstance(array $options) {
    return $this->originalService->getInstance($options);
  }

  /**
   * @inheritDoc
   */
  public function mail($module, $key, $to, $langcode, $params = [], $reply = NULL, $send = TRUE) {
    $email_handler = \Drupal::service('easy_email.handler');
    /** @var \Drupal\easy_email_override\Entity\EmailOverrideInterface[] $email_overrides */
    $email_overrides = \Drupal::entityTypeManager()
      ->getStorage('easy_email_override')
      ->loadByProperties([
        'module' => $module,
        'key' => $key,
      ]);
    if (!empty($email_overrides)) {
      // If we find more than one override for a given module/key combo, we'll send them all.
      // Not sure if that will be useful, but perhaps.
      foreach ($email_overrides as $email_override) {
        $email = $email_handler->createEmail([
          'type' => $email_override->getEasyEmailType(),
        ]);
        $param_map = $email_override->getParamMap();
        foreach ($param_map as $pm) {
          $email->set($pm['destination'], $params[$pm['source']]);
        }
        $result = $email_handler->sendEmail($email);
        $send = FALSE;
      }
    }
    $message = $this->originalService->mail($module, $key, $to, $langcode, $params, $reply, $send);
    if (!isset($message['result']) && !empty($email_overrides)) {
      $message['result'] = TRUE;
    }

    return $message;
  }

}
